@@ -35,6 +35,9 @@ typedef struct TickitWindow TickitWindow;
 
 typedef struct Tickit Tickit;
 
+/* Opaque struct pointers required but not part of official API */
+typedef struct TickitTermDriver TickitTermDriver;
+
 typedef struct {
   int top;
   int left;
@@ -52,6 +55,12 @@ typedef enum {
   TICKIT_BIND_DESTROY = 1<<2,
 } TickitBindFlags;
 
+typedef enum {
+  TICKIT_CTL_USE_ALTSCREEN = 1,
+
+  TICKIT_N_CTLS
+} TickitCtl;
+
 typedef enum {
   TICKIT_CURSORSHAPE_BLOCK = 1,
   TICKIT_CURSORSHAPE_UNDER,
@@ -77,10 +86,10 @@ typedef enum {
 } TickitMaybeBool;
 
 typedef enum {
-  TICKIT_PEN_FG,         /* colour */
+  TICKIT_PEN_FG = 1,     /* colour */
   TICKIT_PEN_BG,         /* colour */
   TICKIT_PEN_BOLD,       /* bool */
-  TICKIT_PEN_UNDER,      /* bool: TODO - number? */
+  TICKIT_PEN_UNDER,      /* number */
   TICKIT_PEN_ITALIC,     /* bool */
   TICKIT_PEN_REVERSE,    /* bool */
   TICKIT_PEN_STRIKE,     /* bool */
@@ -91,10 +100,20 @@ typedef enum {
 } TickitPenAttr;
 
 typedef enum {
-  TICKIT_PENTYPE_BOOL,
-  TICKIT_PENTYPE_INT,
-  TICKIT_PENTYPE_COLOUR,
-} TickitPenAttrType;
+  TICKIT_PEN_UNDER_NONE,
+  TICKIT_PEN_UNDER_SINGLE,
+  TICKIT_PEN_UNDER_DOUBLE,
+  TICKIT_PEN_UNDER_WAVY,
+
+  TICKIT_N_PEN_UNDERS
+} TickitPenUnderline;
+
+typedef enum {
+  TICKIT_RUN_DEFAULT = 0,
+  TICKIT_RUN_ONCE    = 1<<0,
+  TICKIT_RUN_NOHANG  = 1<<1,
+  TICKIT_RUN_NOSETUP = 1<<2,
+} TickitRunFlags;
 
 typedef enum {
   /* This is part of the API so additions must go at the end only */
@@ -119,6 +138,24 @@ typedef enum {
   TICKIT_TERM_MOUSEMODE_MOVE,
 } TickitTermMouseMode;
 
+typedef enum {
+  TICKIT_TYPE_NONE,
+  TICKIT_TYPE_BOOL,
+  TICKIT_TYPE_INT,
+  TICKIT_TYPE_STR,
+  TICKIT_TYPE_COLOUR, // currently unused except by pen
+} TickitType;
+
+typedef enum {
+  TICKIT_WINCTL_STEAL_INPUT = 1,
+  TICKIT_WINCTL_FOCUS_CHILD_NOTIFY,
+  TICKIT_WINCTL_CURSORVIS,
+  TICKIT_WINCTL_CURSORBLINK,
+  TICKIT_WINCTL_CURSORSHAPE,
+
+  TICKIT_N_WINCTLS
+} TickitWindowCtl;
+
 typedef enum {
   TICKIT_WINDOW_HIDDEN      = 1<<0,
   TICKIT_WINDOW_LOWEST      = 1<<1,
@@ -136,6 +173,13 @@ enum {
   TICKIT_MOD_CTRL  = 0x04,
 };
 
+/* back-compat name */
+typedef enum {
+  TICKIT_PENTYPE_BOOL   = TICKIT_TYPE_BOOL,
+  TICKIT_PENTYPE_INT    = TICKIT_TYPE_INT,
+  TICKIT_PENTYPE_COLOUR = TICKIT_TYPE_COLOUR,
+} TickitPenAttrType;
+
 /*
  * Secondary structures
  */
@@ -331,6 +375,19 @@ TickitTerm *tickit_term_new(void);
 TickitTerm *tickit_term_new_for_termtype(const char *termtype);
 void tickit_term_destroy(TickitTerm *tt);
 
+// Internal use only for now but consider making it public API
+struct TickitTermBuilder {
+  const char *termtype;
+
+  TickitTermDriver *driver;
+
+  const struct TickitTerminfoHook {
+    const char *(*getstr)(const char *name, const char *value, void *data);
+    void         *data;
+  } *ti_hook;
+};
+TickitTerm *tickit_term_build(const struct TickitTermBuilder *builder);
+
 TickitTerm *tickit_term_ref(TickitTerm *tt);
 void        tickit_term_unref(TickitTerm *tt);
 
@@ -408,25 +465,20 @@ void tickit_term_emit_mouse(TickitTerm *tt, TickitMouseEventInfo *info);
 const char *tickit_term_ctlname(TickitTermCtl ctl);
 TickitTermCtl tickit_term_lookup_ctl(const char *name);
 
+TickitType tickit_term_ctltype(TickitTermCtl ctl);
+
 /* String handling utilities */
 
 int tickit_utf8_seqlen(long codepoint);
-DEPRECATED int tickit_string_seqlen(long codepoint); /* use tickit_utf8_seqlen() */
 
 /* Does NOT NUL-terminate the buffer */
 size_t tickit_utf8_put(char *str, size_t len, long codepoint);
-DEPRECATED size_t tickit_string_putchar(char *str, size_t len, long codepoint); /* use tickit_utf8_put() */
 
 size_t tickit_utf8_count(const char *str, TickitStringPos *pos, const TickitStringPos *limit);
 size_t tickit_utf8_countmore(const char *str, TickitStringPos *pos, const TickitStringPos *limit);
 size_t tickit_utf8_ncount(const char *str, size_t len, TickitStringPos *pos, const TickitStringPos *limit);
 size_t tickit_utf8_ncountmore(const char *str, size_t len, TickitStringPos *pos, const TickitStringPos *limit);
 
-DEPRECATED size_t tickit_string_count(const char *str, TickitStringPos *pos, const TickitStringPos *limit);
-DEPRECATED size_t tickit_string_countmore(const char *str, TickitStringPos *pos, const TickitStringPos *limit);
-DEPRECATED size_t tickit_string_ncount(const char *str, size_t len, TickitStringPos *pos, const TickitStringPos *limit);
-DEPRECATED size_t tickit_string_ncountmore(const char *str, size_t len, TickitStringPos *pos, const TickitStringPos *limit);
-
 // Some convenient mutators for TickitStringPos structs
 
 static inline void tickit_stringpos_zero(TickitStringPos *pos) {
@@ -463,10 +515,6 @@ static inline void tickit_stringpos_limit_columns(TickitStringPos *pos, int colu
   pos->columns = columns;
 }
 
-DEPRECATED int    tickit_string_mbswidth(const char *str);
-DEPRECATED int    tickit_string_byte2col(const char *str, size_t byte);
-DEPRECATED size_t tickit_string_col2byte(const char *str, int col);
-
 int    tickit_utf8_mbswidth(const char *str);
 int    tickit_utf8_byte2col(const char *str, size_t byte);
 size_t tickit_utf8_col2byte(const char *str, int col);
@@ -633,12 +681,20 @@ void tickit_window_take_focus(TickitWindow *win);
 bool tickit_window_is_focused(const TickitWindow *win);
 void tickit_window_set_focus_child_notify(TickitWindow *win, bool notify);
 
+bool tickit_window_getctl_int(TickitWindow *win, TickitWindowCtl ctl, int *value);
+bool tickit_window_setctl_int(TickitWindow *win, TickitWindowCtl ctl, int value);
+
 bool tickit_window_is_steal_input(const TickitWindow *win);
 void tickit_window_set_steal_input(TickitWindow *win, bool steal);
 
+const char *tickit_window_ctlname(TickitWindowCtl ctl);
+TickitWindowCtl tickit_window_lookup_ctl(const char *name);
+
+TickitType tickit_window_ctltype(TickitWindowCtl ctl);
+
 /* Main object */
 
-typedef int TickitCallbackFn(Tickit *t, TickitEventFlags flags, void *user);
+typedef int TickitCallbackFn(Tickit *t, TickitEventFlags flags, void *info, void *user);
 
 Tickit *tickit_new_for_term(TickitTerm *tt);
 Tickit *tickit_new_stdio(void);
@@ -653,11 +709,33 @@ TickitWindow *tickit_get_rootwin(Tickit *t);
 void tickit_run(Tickit *t);
 void tickit_stop(Tickit *t);
 
-int tickit_timer_after_msec(Tickit *t, int msec, TickitBindFlags flags, TickitCallbackFn *fn, void *user);
-int tickit_timer_after_tv(Tickit *t, const struct timeval *after, TickitBindFlags flags, TickitCallbackFn *fn, void *user);
-void tickit_timer_cancel(Tickit *t, int id);
+void tickit_tick(Tickit *t, TickitRunFlags flags);
 
-int tickit_later(Tickit *t, TickitBindFlags flags, TickitCallbackFn *fn, void *user);
+bool tickit_getctl_int(Tickit *tt, TickitCtl ctl, int *value);
+bool tickit_setctl_int(Tickit *tt, TickitCtl ctl, int value);
+
+const char *tickit_ctlname(TickitCtl ctl);
+TickitCtl tickit_lookup_ctl(const char *name);
+
+TickitType tickit_ctltype(TickitCtl ctl);
+
+/* TODO: Consider OUT and HUP conditions too */
+void *tickit_watch_io_read(Tickit *t, int fd, TickitBindFlags flags, TickitCallbackFn *fn, void *user);
+
+void *tickit_watch_timer_after_msec(Tickit *t, int msec, TickitBindFlags flags, TickitCallbackFn *fn, void *user);
+void *tickit_watch_timer_after_tv(Tickit *t, const struct timeval *after, TickitBindFlags flags, TickitCallbackFn *fn, void *user);
+
+void *tickit_watch_timer_at_epoch(Tickit *t, time_t at, TickitBindFlags flags, TickitCallbackFn *fn, void *user);
+void *tickit_watch_timer_at_tv(Tickit *t, const struct timeval *at, TickitBindFlags flags, TickitCallbackFn *fn, void *user);
+
+void *tickit_watch_later(Tickit *t, TickitBindFlags flags, TickitCallbackFn *fn, void *user);
+
+void tickit_watch_cancel(Tickit *t, void *watch);
+
+/* Entirely undocumented right now */
+void tickit_hook_terminfo(Tickit *t,
+    const char *(*getstr)(const char *name, const char *value, void *data),
+    void         *data);
 
 /* Debug support */
 
